package helpdeskClient;


import java.net.*;
import java.io.*;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import javax.swing.DefaultListModel;
import javax.swing.JList;

public class Client  
{

	private ObjectInputStream sInput;		
	private ObjectOutputStream sOutput;		
	private Socket socket;

	private SimpleDateFormat sdf;
	private ClientGUI cg;
	
	private String server, username, passwd;
	String[] conversation = {""};

	private int port;

	Client(String server, int port, String username, String passwd, ClientGUI cg) 
	{
		this.server = server;
		this.port = port;
		this.username = username;
		this.passwd = passwd;
		sdf = new SimpleDateFormat("HH:mm:ss");
		this.cg = cg;
	}
	
	public boolean start() 
	{
		try 
		{
			socket = new Socket(server, port);
		} 
		catch(Exception ec) 
		{
			display("Error connectiong to server:" + ec);
			return false;
		}

		try
		{
			sInput  = new ObjectInputStream(socket.getInputStream());
			sOutput = new ObjectOutputStream(socket.getOutputStream());
		}
		catch (IOException eIO) 
		{
			display("Exception creating new Input/output Streams: " + eIO);
			return false;
		}

		new GoClient().start();
		try
		{
			FileInputStream fis1 = new FileInputStream("src/private");
			byte[] privByte = new byte[fis1.available()];
			fis1.read(privByte);
			fis1.close();
			RSAPrivateKey priv = (RSAPrivateKey) KeyFactory.getInstance("RSA").generatePrivate(new PKCS8EncodedKeySpec(privByte));
			byte[] myKey = new byte[8];
			for (int i = 0; i < 7; i ++)
			{
				myKey[i] = priv.getEncoded()[i];
			}
			Key secret = new SecretKeySpec(myKey, "DES");
			String toSend= username + ", " + passwd;
			Cipher cipher = Cipher.getInstance("DES/ECB/PKCS5Padding");
			cipher.init(Cipher.ENCRYPT_MODE, secret);
			byte[] ct = cipher.doFinal(toSend.getBytes());
			sOutput.writeObject(ct);
		}
		catch (GeneralSecurityException e)
		{
			e.printStackTrace();
		}
		catch (IOException eIO) 
		{
			display("Exception doing login : " + eIO);
			disconnect();
			return false;
		}
		String msg = "Connection accepted " + socket.getInetAddress() + ":" + socket.getPort();
		display(msg);
		return true;
	}
	
	String getUsername()
	{
		return username;
	}

	String[] getConversation()
	{
		return conversation;
	}
	
	void setUsername(String u)
	{
		this.username = u;
	}
	
	
	private void display(String msg) 
	{
		cg.append(msg + "\n");		
	}
	
	void sendMessage(SystemMessage msg) 
	{
		try 
		{
			sOutput.writeObject(msg);
		}
		catch(IOException e)
		{
			display("Exception writing to server: " + e);
		}
	}

	private void disconnect() 
	{
		try 
		{ 
			if(sInput != null) sInput.close();
		}
		catch(Exception e) {} 
		try 
		{
			if(sOutput != null) sOutput.close();
		}
		catch(Exception e) {} 
        try
        {
			if(socket != null) socket.close();
		}
		catch(Exception e) {} 
		
		cg.connectionFailed();
	}

	class GoClient extends Thread 
	{
		public void run() 
		{
			while(true) 
			{
				try 
				{
					SystemMessage cm = (SystemMessage) sInput.readObject();
					String msg = "";
					try
					{
						FileInputStream fis1 = new FileInputStream("src/private");
						byte[] privByte = new byte[fis1.available()];
						fis1.read(privByte);
						fis1.close();
						RSAPrivateKey priv = (RSAPrivateKey) KeyFactory.getInstance("RSA").generatePrivate(new PKCS8EncodedKeySpec(privByte));
						byte[] myKey = new byte[8];
						for (int i = 0; i < 7; i ++)
						{
							myKey[i] = priv.getEncoded()[i];
						}
						Key secret = new SecretKeySpec(myKey, "DES");
						Cipher cipher = Cipher.getInstance("DES/ECB/PKCS5Padding");
						cipher.init(Cipher.DECRYPT_MODE, secret);
						byte[] back = cipher.doFinal(cm.getMessage());
						msg = new String(back);
					}
					catch (GeneralSecurityException e)
					{
						e.printStackTrace();
						break;
					}
					int type = cm.getType();
					String to = cm.getToWho();
					if (type==SystemMessage.LOGOUT)
					{
						if (cm.getFromWho().equals("server"))
						{
							cg.clear();
							display(msg);
							break;
						}
						else
						{
							display(msg);						
						}
					}
					else if (type==SystemMessage.LOGIN)
					{
						display("successfully log in as " + to);
						if (!to.substring(0, 2).equalsIgnoreCase("HD"))
						{
							if (msg!=null)
							{
								display(msg + " is here to help you.");
							}
						}
						else
						{
							cg.getTransfer().setEnabled(true);
							cg.getUpgrade().setEnabled(true);
							cg.getTransfer().setToolTipText("transfer an anonymous user to another help desk user");
							cg.getUpgrade().setToolTipText("upgrade an anonymous user to help desk user");
						}
						cg.getAccountlabel().setText("You are logged in as " + to);
						cg.setTitle("Help Desk System ------" + to);
						cg.getClient().setUsername(to);
					}
					else if (type==SystemMessage.TALKTOWHO)
					{
						if (!msg.equals(""))
						{
							if (!username.substring(0, 2).equalsIgnoreCase("HD"))
							{
								display("now " + msg + " is here to help you.");
							}
							
							conversation = msg.split(",");
							String[] conv = conversation;
							JList<String> talkto = cg.getTalkTo();
							DefaultListModel<String> listModel = new DefaultListModel<String>();
							talkto.setModel(listModel);
							for (int i = 0; i < conv.length; i++)
							{
								listModel.addElement(conv[i]);
							}
							talkto.setModel(listModel);
						}
						else if (msg.equals(""))
						{
							JList<String> talkto = cg.getTalkTo();
							DefaultListModel<String> listModel = new DefaultListModel<String>();
							talkto.setModel(listModel);
							cg.append(sdf.format(new Date()) + " No one you can talk to! Please try again later.\n");
						}

					}
					else if (type == SystemMessage.UPGRADE)
					{
						if (to.equals(cg.getClient().username))
						{
							UpgradeWindow upgrade = new UpgradeWindow(cg, true);
							while (!upgrade.isDone());
							cg.getClient().sendMessage(new SystemMessage(to, "", SystemMessage.UPGRADE, upgrade.getUsername() + "," + upgrade.getPasswd()));
						}
						else 
						{
							cg.getAccountlabel().setText("You are logged in as " + to);
							cg.setTitle("Help Desk System ------" + to);
							cg.getClient().setUsername(to);
							cg.getTransfer().setEnabled(true);
							cg.getUpgrade().setEnabled(true);
							cg.getTransfer().setToolTipText("transfer an anonymous user to another help desk user");
							cg.getUpgrade().setToolTipText("upgrade an anonymous user to help desk user");
							cg.append(sdf.format(new Date()) + " Congratulations! " + cg.getClient(). getUsername() + " you are a Help Desk User, you can help others now\n");
						}
					}
					else
					{
						cg.append(sdf.format(new Date()) + " " + msg);
					}
				}
				catch(IOException e) {
					display("Server has close the connection: " + e);
					cg.connectionFailed();
					break;
				}
				catch(ClassNotFoundException e2) {
				}
			}
			cg.connectionFailed();
		}
	}
}

