package helpdeskClient;


import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

public class ClientGUI extends JFrame implements ActionListener 
{

	private static final long serialVersionUID = 1L;
	private JLabel accountlabel,talkToLabel, mslabel, unLabel, pwLabel;
	private JTextField un, message;
	
	private JList<String> talkTo;
	private JPasswordField pw;
	private JTextField tfServer, tfPort;
	private JButton loginout, whoIsIn, send, transfer, upgrade;
	private JTextArea ta;
	private boolean connected;
	
	private JScrollPane sp;
	private Client client;
	private int defaultPort;
	private String defaultHost;

	ClientGUI(String host, int port) 
	{

		super("Help Desk Client");
		defaultPort = port;
		defaultHost = host;
		JPanel northPanel = new JPanel(new GridLayout(4,1));
		JPanel serverAndPort = new JPanel(new GridLayout(1,5, 1, 3));
		tfServer = new JTextField(host);
		tfPort = new JTextField("" + port);
		tfPort.setHorizontalAlignment(SwingConstants.RIGHT);

		serverAndPort.add(new JLabel("Server Address:  "));
		serverAndPort.add(tfServer);
		serverAndPort.add(new JLabel("Port Number:  "));
		serverAndPort.add(tfPort);
		serverAndPort.add(new JLabel(""));
		northPanel.add(serverAndPort);

		accountlabel = new JLabel("Enter your account infomation below", SwingConstants.CENTER);
		northPanel.add(accountlabel);
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		unLabel = new JLabel("Username: ");
		pwLabel = new JLabel("Password: ");
		un = new JTextField();
		pw = new JPasswordField();
		infoPanel.add(unLabel);
		infoPanel.add(un);
		infoPanel.add(pwLabel);
		infoPanel.add(pw);
		northPanel.add(infoPanel);
		add(northPanel, BorderLayout.NORTH);
		
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		talkToLabel = new JLabel("Talk To: ",  SwingConstants.LEFT);
		talkTo = new JList<String>();
		left.add(talkToLabel);
		sp = new JScrollPane(talkTo);
		sp.setMinimumSize(new Dimension(100, 180));
		left.add(sp);
		
		JPanel right = new JPanel();
		right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
		ta = new JTextArea("Welcome to the Help Desk System\n", 80, 80);
		ta.setEditable(false);
		right.add(new JScrollPane(ta));
		
		mslabel = new JLabel("Enter your message below: ",  SwingConstants.LEFT);
		message = new JTextField();
		right.add(mslabel);
		right.add(message);
		

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.X_AXIS));
		centerPanel.add(left);
		centerPanel.add(Box.createRigidArea(new Dimension(20, 0)));
		centerPanel.add(right);
		add(centerPanel, BorderLayout.CENTER);

		loginout = new JButton("Login");
		loginout.addActionListener(this);
		whoIsIn = new JButton("Who is in");
		whoIsIn.addActionListener(this);
		whoIsIn.setEnabled(false);		
		send = new JButton("Send");
		send.addActionListener(this);
		send.setEnabled(false);
		transfer = new JButton("Transfer");
		transfer.addActionListener(this);
		transfer.setEnabled(false);
		upgrade = new JButton("Upgrade");
		upgrade.addActionListener(this);
		upgrade.setEnabled(false);
		loginout.setToolTipText("login using the hd account or as anonymous user");
		whoIsIn.setToolTipText("you need to login to update the conversation list");
		send.setToolTipText("you need to login to send message");
		transfer.setToolTipText("you need to login as help desk user");
		upgrade.setToolTipText("you need to login as help desk user");
		
		JPanel southPanel = new JPanel();
		southPanel.add(loginout);
		southPanel.add(whoIsIn);
		southPanel.add(send);
		southPanel.add(transfer);
		southPanel.add(upgrade);
		
		add(southPanel, BorderLayout.SOUTH);

		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(700, 600);
		setVisible(true);
		this.setTitle("Help Desk System ------ not logged in");
		loginout.requestFocus();

	}

	void append(String str) 
	{
		ta.append(str);
		ta.setCaretPosition(ta.getText().length() - 1);
	}
	
	public void clear()
	{
		ta.setText("Welcome to the Help Desk System\n");
	}
	
	void connectionFailed() 
	{
		whoIsIn.setEnabled(false);
		send.setEnabled(false);
		transfer.setEnabled(false);
		upgrade.setEnabled(false);
		loginout.setText("login");
		accountlabel.setText("Enter your account infomation below");
		unLabel.setVisible(true);
		pwLabel.setVisible(true);
		un.setVisible(true);
		pw.setVisible(true);
		un.setText("");
		pw.setText("");
		tfPort.setText("" + defaultPort);
		tfServer.setText(defaultHost);
		tfServer.setEditable(true);
		tfPort.setEditable(true);
		message.removeActionListener(this);
		connected = false;
		loginout.setToolTipText("login using the hd account or as anonymous user");
		whoIsIn.setToolTipText("you need to login to update the conversation list");
		send.setToolTipText("you need to login to send message");
		transfer.setToolTipText("you need to login as help desk user");
		upgrade.setToolTipText("you need to login as help desk user");
		this.setTitle("Help Desk System ------ not logged in");

	}
		
	@SuppressWarnings("deprecation")
	public void actionPerformed(ActionEvent e) 
	{
		Object o = e.getSource();
		if(o == loginout && connected) 
		{
			client.sendMessage(new SystemMessage(client.getUsername(), "", SystemMessage.LOGOUT, ""));
			return;
		}
		if(o == whoIsIn) 
		{
			client.sendMessage(new SystemMessage(client.getUsername(), "", SystemMessage.TALKTOWHO, ""));	
			return;
		}
		
		if(o == transfer)
		{
			if (client.conversation[0].equals(""))
			{
				append("No one in the conversation list, cannot transfer!\n");
				return;
			}
			TransferWindow newwindow = new TransferWindow(this, true, client);
			newwindow.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
			newwindow.setTitle("Transfer an anonymous user to Help Desk User");
			newwindow.pack();
			newwindow.setVisible(true);
			String[] t = newwindow.getTransferArray();
			if (!newwindow.getSelected())
				return;
			client.sendMessage(new SystemMessage(client.getUsername(), "", SystemMessage.TRANSFER, t[0] + "," + t[1]));
		}
		
		if (o == upgrade)
		{
			if (talkTo.getSelectedValue() == null || talkTo.getSelectedValue().substring(0, 2).equalsIgnoreCase("HD"))
			{
				append("Please select a Anonymous user to upgrade\n");
				return;
			}
			int re = JOptionPane.showConfirmDialog(this, "Confirm to upgrade "+ talkTo.getSelectedValue(), 
					"Confirm to upgrade "+ talkTo.getSelectedValue(), JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (re != 0)
				return;
			client.sendMessage(new SystemMessage(client.getUsername(), talkTo.getSelectedValue(), SystemMessage.UPGRADE, ""));	
			return;
		}

		if(connected && (o == message || o == send)) 
		{
			String towho = talkTo.getSelectedValue();
			String msg = message.getText();
			if (msg.equals(""))
			{
				append("Cannot send empty message\n");
				return;
			}
			else if (towho == null)
			{
				append("Who do you want to talk to?\n");
				return;
			}
			client.sendMessage(new SystemMessage(client.getUsername(), towho, SystemMessage.MESSAGE, msg));				
			message.setText("");
			
			return;
		}
		
		if(o == loginout && !connected) 
		{
			String username = un.getText().trim();
			String passwd = pw.getText().trim();
			loginout.setText("logout");
			String server = tfServer.getText().trim();
			if(server.length() == 0)
				return;
			String portNumber = tfPort.getText().trim();
			if(portNumber.length() == 0)
				return;
			int port = 0;
			try 
			{
				port = Integer.parseInt(portNumber);
			}
			catch(Exception en) 
			{
				return;
			}

			client = new Client(server, port, username, passwd, this);
			if(!client.start()) 
				return;
			
			unLabel.setVisible(false);
			pwLabel.setVisible(false);
			un.setVisible(false);
			pw.setVisible(false);
			connected = true;
			whoIsIn.setEnabled(true);
			send.setEnabled(true);
			sp.setMinimumSize(new Dimension(100, 180));
			tfServer.setEditable(false);
			tfPort.setEditable(false);
			
			loginout.setToolTipText("logout the account");
			whoIsIn.setToolTipText("update the conversation list");
			send.setToolTipText("send the message");

			message.addActionListener(this);
		}
	}
	
	JList<String> getTalkTo()
	{
		return talkTo;
	}
	
	JLabel getAccountlabel()
	{
		return accountlabel;
	}

	Client getClient()
	{
		return client;
	}
	
	JButton getTransfer()
	{
		return transfer;
	}
	
	JButton getUpgrade()
	{
		return upgrade;
	}
	
	public static void main(String[] args) 
	{
		new ClientGUI("localhost", 9525);
	}
}

