package helpdeskClient;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

public class TransferWindow extends JDialog implements ActionListener
{
	private static final long serialVersionUID = 1L;
	private DefaultListModel<String> listUSModel;
	private DefaultListModel<String> listHDModel;
	
	private JLabel leftL, rightL;
	private JList<String> transferUSList, transferHDList;
	private JScrollPane listUSScrollPane, listHDScrollPane;
	private JButton okay, cancel;
	private String[] transferArray;
	private boolean selected;
	
	public String[] getTransferArray()
	{
		return transferArray;
	}
	
	public boolean getSelected()
	{
		return selected;
	}
	
	public TransferWindow(JFrame frame, boolean modal, Client client)
	{
		super(frame, modal);
		listUSModel = new DefaultListModel<String>();
		listHDModel = new DefaultListModel<String>();
		for (int i = 0; i < client.conversation.length; i++)
		{
			if (client.conversation[i].substring(0, 2).equalsIgnoreCase("HD"))
				listHDModel.addElement(client.conversation[i]);
			else
				listUSModel.addElement(client.conversation[i]);
		}
		transferArray = new String[2];
		selected = false;
		this.setLayout(new BorderLayout());
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		JPanel right = new JPanel();
		right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
		leftL = new JLabel("choose user:          ");
		rightL = new JLabel("choose Help Desk User:");
		
		
		transferUSList = new JList<String>(listUSModel);
		transferHDList = new JList<String>(listHDModel);
		listUSScrollPane = new JScrollPane(transferUSList);
		listHDScrollPane = new JScrollPane(transferHDList);
		listUSScrollPane.setPreferredSize(listHDScrollPane.getPreferredSize());
		
		transferUSList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		transferUSList.setSelectedIndex(0);
		transferHDList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		transferHDList.setSelectedIndex(0);
		transferUSList.setVisibleRowCount(10);
		transferHDList.setVisibleRowCount(10);
		
		left.add(leftL);
		left.add(listUSScrollPane);
		right.add(rightL);
		right.add(listHDScrollPane);

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.X_AXIS));
		centerPanel.add(left);
		centerPanel.add(Box.createRigidArea(new Dimension(20, 0)));
		centerPanel.add(right);
		
		JPanel bottom = new JPanel();
		bottom.setLayout(new FlowLayout(FlowLayout.CENTER));
		okay = new JButton("OK");
		cancel = new JButton("Cancel");
		okay.addActionListener(this);
		cancel.addActionListener(this);
		okay.setPreferredSize(cancel.getPreferredSize());
		bottom.add(okay);
		bottom.add(cancel);
		
		
		setLocationRelativeTo(frame);
		add(centerPanel, BorderLayout.CENTER);
		add(bottom, BorderLayout.SOUTH);
		
	}
		
	@Override
	public void actionPerformed(ActionEvent ar)
	{
		if (okay == ar.getSource())
		{
			transferArray[0] = transferUSList.getSelectedValue();
			transferArray[1] = transferHDList.getSelectedValue();
			if (transferArray[0] == null || transferArray[1] == null)
			{
				selected = false;
				JOptionPane.showMessageDialog(this, "Please choose one user to transfer and "
						+ "one HD user to transfer to!", "Error", JOptionPane.ERROR_MESSAGE);
			}
			else 
			{
				selected = true;
				setVisible(false);
			}
		}
		if (cancel == ar.getSource())
		{
			selected = false;
			setVisible(false);
		}
			
	}
	
}
