package helpdeskServer;

import java.io.*;
import java.net.*;
import javax.crypto.*;
import java.security.*;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.text.SimpleDateFormat;
import java.util.*;

public class Server
{
	static int uniqueId;
	ArrayList<ClientThread> al;
	private ServerGUI sg;
	SimpleDateFormat sdf;
	private int port;
	private boolean keepGoing;
	
	PrintWriter pw1;

	Map<String, String> accountMap = new HashMap<String, String>(10);

	List<List<String>> online = new ArrayList<List<String>>();

	List<List<Integer>> idlist = new ArrayList<List<Integer>>();
	List<Integer> holdCount = new ArrayList<Integer>();

	public Server(int port, ServerGUI sg) throws FileNotFoundException
	{

		this.sg = sg;
		this.port = port;
		sdf = new SimpleDateFormat("HH:mm:ss");
		pw1 = new PrintWriter("ConversationLog");
		pw1.printf("Help Desk. Starts at %s\n", new SimpleDateFormat("YY-MM-DD-HH:mm:ss").format(new Date()));
		al = new ArrayList<ClientThread>();
	}

	public void start()
	{
		keepGoing = true;
		try
		{
			ServerSocket serverSocket = new ServerSocket(port);

			FileInputStream fis1 = new FileInputStream("src/private");
			byte[] privByte = new byte[fis1.available()];
			fis1.read(privByte);
			fis1.close();
			RSAPrivateKey priv = (RSAPrivateKey) KeyFactory.getInstance("RSA").generatePrivate(new PKCS8EncodedKeySpec(privByte));

			FileInputStream fis = new FileInputStream("src/account");
			byte[] in1 = new byte[fis.available()/2];
			byte[] in2 = new byte[fis.available() - fis.available()/2];
			fis.read(in1);
			fis.read(in2);
			fis.close();
		
			Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
			cipher.init(Cipher.DECRYPT_MODE, priv);
			byte[] back1 = cipher.doFinal(in1);
			byte[] back2 = cipher.doFinal(in2);
			
			String[] line1= new String(back1).split("\n");
			for (int i = 0; i < line1.length; i ++)
			{
				String[] account = line1[i].split(",");
				accountMap.put(account[0], account[1]);
			}
			String[] line2= new String(back2).split("\n");
			
			for (int i = 0; i < line2.length; i ++)
			{
				String[] account = line2[i].split(",");
				accountMap.put(account[0], account[1]);
			}
			
			while (keepGoing)
			{
				display("Server waiting for Clients on port " + port + ".");

				Socket socket = serverSocket.accept(); // accept connection
				if (!keepGoing)
				{
					break;
				}
				uniqueId = uniqueId+1;
				ClientThread t = new ClientThread(this, socket); // make a thread of it
				
				t.start();
			}
			try
			{
				serverSocket.close();
				for (int i = 0; i < al.size(); ++i)
				{
					ClientThread tc = al.get(i);
					try
					{
						tc.sInput.close();
						tc.sOutput.close();
						tc.socket.close();
					} catch (IOException ioE)
					{
						ioE.printStackTrace();
					}
				}
			} catch (Exception e)
			{
				display("Exception closing the server and clients: " + e);
			}
		}
		catch (IOException e)
		{
			String msg = sdf.format(new Date()) + " Exception on new ServerSocket: " + e + "\n";
			display(msg);
		} catch (NoSuchAlgorithmException e1)
		{
			e1.printStackTrace();
		} catch (NoSuchPaddingException e1)
		{
			e1.printStackTrace();
		} catch (InvalidKeySpecException e1)
		{
			e1.printStackTrace();
		} catch (InvalidKeyException e1)
		{
			e1.printStackTrace();
		} catch (IllegalBlockSizeException e1)
		{
			e1.printStackTrace();
		} catch (BadPaddingException e1)
		{
			e1.printStackTrace();
		}
	}

	static int getUniqueId()
	{
		return uniqueId;
	}

	void stop()
	{
		keepGoing = false;
		pw1.close();
		try
		{
			new Socket("localhost", port);
		} catch (Exception e)
		{
		}
	}


	void display(String msg)
	{
		String time = sdf.format(new Date()) + " " + msg;
		sg.appendEvent(time + "\n");
	}


	synchronized void removeId(int id)
	{
		for (int i = 0; i < al.size(); ++i)
		{
			ClientThread ct = al.get(i);
			if (ct.id == id)
			{
				al.remove(i);
				return;
			}
		}
	}

	synchronized int[] addUser(String username, int id)
	{
		int[] at = new int[2];
		if (username.substring(0, 2).equalsIgnoreCase("hd"))
		{
			if (online.size() == 0 || idlist.get(0).get(0) != 0)
			{
				List<String> toAdd = new ArrayList<String>();
				toAdd.add(username);
				online.add(toAdd);
				holdCount.add(0);
				
				List<Integer> toPut = new ArrayList<Integer>();
				toPut.add(id);
				idlist.add(toPut);
				at[0] = idlist.size() - 1;
				return at;
			}
			else
			{
				online.get(0).set(0, username);
				idlist.get(0).set(0, id);
				
				for (int k = 1; k < idlist.get(0).size(); k++)
				{
					int target = idlist.get(0).get(k);
					if (target != 0)
					{
						getThread(target).updateConversation();
					}
				}
				at[0] = 0;
				at[1] = 0;
				return at;
			}
		}
		else
		{
			if(idlist.size() == 0 || idlist.get(0).get(0) == 0)
			{
				if (online.size() == 0)
				{
					List<String> toAdd = new ArrayList<String>();
					toAdd.add("");
					toAdd.add(username);
					online.add(toAdd);
					
					List<Integer> toPut = new ArrayList<Integer>();
					toPut.add(0);
					toPut.add(id);
					idlist.add(toPut);
					
					holdCount.add(1);
					at[0] = 0;
					at[1] = 1;
					return at;
				}
				else 
				{
					online.get(0).add(username);
					idlist.get(0).add(id);
					holdCount.set(0, holdCount.get(0) + 1);
					
					at[0]=0;
					at[1] = online.get(0).indexOf(username);
					return at;
				}
			}
			
			else
			{
				int lowest = Integer.MAX_VALUE, oneToAdd = 0;
				
				for (int i = 0; i < idlist.size(); i++)
				{
					if (idlist.get(i).size() < lowest)
					{
						lowest = idlist.get(i).size();
						oneToAdd = i;
					}
				}
	
				online.get(oneToAdd).add(username);
				idlist.get(oneToAdd).add(id);
				holdCount.set(oneToAdd, holdCount.get(oneToAdd) + 1);
				at[0] = oneToAdd;
				at[1] = idlist.get(oneToAdd).size()-1;
				return at;
			}
		}
	}

	ArrayList<ClientThread> getAl()
	{
		return al;
	}

	void setAl(ArrayList<ClientThread> al)
	{
		this.al = al;
	}

	synchronized int[] transferUser(String user, String Hd)
	{
		int[] userPosition = new int[2];
		int[] HdPosition = new int[2];
		int[] at = new int[2];
		for (int i = 0; i < idlist.size(); i++)
		{
			if (online.get(i) == null )
			{
				break;
			}
			if (online.get(i).contains(user))
			{
				userPosition[0] = i;
				userPosition[1] = online.get(i).indexOf(user);
			}
			if (online.get(i).contains(Hd))
			{
				HdPosition[0] = i;
				HdPosition[1] = online.get(i).indexOf(Hd);
			}
		}
		int a = idlist.get(userPosition[0]).get(userPosition[1]);
		deleteUser(user, a);

		online.get(HdPosition[0]).add(user);
		idlist.get(HdPosition[0]).add(a);
		holdCount.set(HdPosition[0], holdCount.get(HdPosition[0]) + 1);
		at[0] = HdPosition[0];
		at[1] = online.get(HdPosition[0]).size()-1;
		return at;
	}

	synchronized void deleteUser(String username, int id)
	{
		if (username.substring(0, 2).equalsIgnoreCase("hd"))
		{
			boolean found = false;
			List<String> forName = new ArrayList<String>();
			List<Integer> forId = new ArrayList<Integer>();
			for (int i = 0; i < idlist.size(); i++)
			{
				if (idlist.get(i).contains(id))
				{
					for (int j = 1; j < idlist.get(i).size(); j++)
					{
						forName.add(online.get(i).get(j));
						forId.add(idlist.get(i).get(j));
					}
					online.remove(i);
					idlist.remove(i);
					holdCount.remove(i);
					found = true;
					i--;
					continue;
				}
				if (found && idlist.size() != 0)
				{
					for (int j = 0; j < idlist.get(i).size(); j++)
					{
						getThread(idlist.get(i).get(j)).position[0]--;
					}
				}
			}
			
			for (int j = 0; j < forId.size(); j++)
			{
				ClientThread ct = getThread(forId.get(j));
				ct.position = addUser(forName.get(j),forId.get(j));
				ct.updateConversation();
			}
		}
		else
		{
			for (int i = 0; i < idlist.size(); i++)
			{
				if (idlist.get(i).contains(id))
				{
					int pos = idlist.get(i).indexOf(id);
					
					idlist.get(i).remove(pos);
					online.get(i).remove(username);
					holdCount.set(i, holdCount.get(i) - 1);
					
					for (int k = pos; k < idlist.get(i).size(); k++)
					{
						getThread(idlist.get(i).get(k)).position[1]--;
					}
				}
			}
		}
	}

	int getId(String username)
	{
		int at = 0;
		for (int i = 0; i < online.size(); i++)
		{
			if(online.get(i).contains(username))
			{
				at = online.get(i).indexOf(username);
				return idlist.get(i).get(at);
			}
		}
		return 0;
	}

	ClientThread getThread(int id)
	{
		for (int k = 0; k < al.size(); k++)
		{
			if (al.get(k).id == id)
				return al.get(k);
		}
		return null;
	}
}
